////////////////////////////////////////////////////////////////////////////////
// Windows note: The StarIO.dll .Net class library depends on the native StarIOPort.dll component.
//               There are seperate 32-bit and 64-bit platform versions of the StarIOPort.dll component.
//               Use Visual Studio's Configuration Manager to switch this project between the platforms.
//               See this project's Post-build event where the appropriate native component is copied to the output directory.
//
// Linux & Mac compile command: "gmcs /out:bin/Sample.exe /r:System.dll /r:System.Data.dll /r:System.Xml.dll /r:bin/StarIO.dll Program.cs"
// Linux & Mac execution command: "mono Sample.exe"
////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

using StarMicronics.StarIO; // added as a reference from the "Dependencies" directory
                                                          // requires StarIOPort.dll, which is copied to the output directory by the Post-Build event

namespace StarMicronics.StarIOSamples.CSharp
{
    internal class Program
    {
        static void Main(string[] args)
        {
            Console.Write("Enter the port name: ");
            string portName = Console.ReadLine();

            Console.Write("Enter the port settings: ");
            string portSettings = Console.ReadLine();

            bool run = true;
            while (run == true)
            {
                Console.WriteLine();

                Console.WriteLine("[1] Write Port");
                Console.WriteLine("[2] Get Status");
                Console.WriteLine("[3] Reset Device");
                Console.WriteLine("[4] Online Status");
                Console.WriteLine("[5] Checked Block");
                Console.WriteLine("[6] Read Port");
                Console.WriteLine("[0] Exit Program");
                Console.Write("Select one of the above options: ");

                uint selection;
                try
                {
                    selection = uint.Parse(Console.ReadLine());
                }
                catch
                {
                    selection = 0;
                }

                switch (selection)
                {
                    case 1:
                        WritePortUsage(portName, portSettings);
                        break;
                    case 2:
                        GetStatusUsage(portName, portSettings);
                        break;
                    case 3:
                        ResetDeviceUsage(portName, portSettings);
                        break;
                    case 4:
                        GetOnlineStatusUsage(portName, portSettings);
                        break;
                    case 5:
                        CheckedBlockUsage(portName, portSettings);
                        break;
                    case 6:
                        ReadPortUsage(portName, portSettings);
                        break;
                    case 0:
                        run = false;
                        break;
                    default:
                        Console.WriteLine("Invalid option");
                        break;
                }
            }
        }

        private static void WritePortUsage(string portName, string portSettings)
        {
            IPort port = null;
            try
            {
                port = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000);

                byte[] command = ASCIIEncoding.ASCII.GetBytes("Hello World.\n");
                uint totalSizeCommunicated = WritePortHelper(port, command);
                if (totalSizeCommunicated != command.Length)
                {
                    Console.WriteLine("Error - could not write out all data.");
                    return;
                }
                else
                {
                    Console.WriteLine("Success");
                }
            }
            catch (PortException e)
            {
                Console.WriteLine("Failure - see stack trace:\n" + e.StackTrace);
            }
            finally
            {
                if (port != null)
                {
                    StarMicronics.StarIO.Factory.I.ReleasePort(port);
                }
            }
        }

        private static void GetStatusUsage(string portName, string portSettings)
        {
            IPort port = null;
            try
            {
                port = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000);
            
                StarPrinterStatus printerStatus = port.GetParsedStatus();
                if (printerStatus.Offline == false)
                {
                    Console.WriteLine("The printer is online");
                }
                else
                {
                    Console.WriteLine("The printer is offline");
                    if (printerStatus.ReceiptPaperEmpty == true)
                    {
                        Console.WriteLine("\tPaper empty");
                    }

                    if (printerStatus.CoverOpen == true)
                    {
                        Console.WriteLine("\tover open");
                    }

                }

                Console.WriteLine("Success");
            }
            catch (PortException e)
            {
                Console.WriteLine("Failure - see stack trace:\n" + e.StackTrace);
            }
            finally
            {
                if (port != null)
                {
                    StarMicronics.StarIO.Factory.I.ReleasePort(port);
                }
            }
        }

        private static void ResetDeviceUsage(string portName, string portSettings)
        {
            IPort port = null;
            try
            {
                port = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000);
            
                port.ResetDevice();
                
                Console.WriteLine("Success");
            }
            catch (PortException e)
            {
                Console.WriteLine("Failure - see stack trace:\n" + e.StackTrace);
            }
            finally
            {
                if (port != null)
                {
                    StarMicronics.StarIO.Factory.I.ReleasePort(port);
                }
            }
        }

        private static void GetOnlineStatusUsage(string portName, string portSettings)
        {
            IPort port = null;
            try
            {
                port = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000);

                bool onlineStatus = port.GetOnlineStatus();
                if (onlineStatus == true)
                {
                    Console.WriteLine("The printer is online");
                }
                else
                {
                    Console.WriteLine("The Printer is offline");
                }

                Console.WriteLine("Success");
            }
            catch (PortException e)
            {
                Console.WriteLine("Failure - see stack trace:\n" + e.StackTrace);
            }
            finally
            {
                if (port != null)
                {
                    StarMicronics.StarIO.Factory.I.ReleasePort(port);
                }
            }
        }

        private static void CheckedBlockUsage(string portName, string portSettings)
        {
            IPort port = null;
            try
            {
                port = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000);
            
                StarPrinterStatus beginningPrinterStatus = port.BeginCheckedBlock();
                if (beginningPrinterStatus.Offline == true)
                {
                    Console.WriteLine("The printer is offline, so printing will not proceed.");
                    return;
                }

                byte[] command = ASCIIEncoding.ASCII.GetBytes("Hello World.\n");
                for (int count = 0; count < 50; count++)
                {
                    uint totalSizeCommunicated = WritePortHelper(port, command);
                    if (totalSizeCommunicated != command.Length)
                    {
                        Console.WriteLine("Error - could not write out all data.");
                        return;
                    }
                }

                StarPrinterStatus endingPrinterStatus = port.EndCheckedBlock();
                if (endingPrinterStatus.Offline == true)
                {
                    Console.WriteLine("The printer is offline, meaning that printing failed.");
                }
                else
                {
                    Console.WriteLine("Success");
                }
            }
            catch (PortException e)
            {
                Console.WriteLine("Failure - see stack trace:\n" + e.StackTrace);
            }
            finally
            {
                if (port != null)
                {
                    StarMicronics.StarIO.Factory.I.ReleasePort(port);
                }
            }
        }

        private static void ReadPortUsage(string portName, string portSettings)
        {
            IPort port = null;
            try
            {
                port = StarMicronics.StarIO.Factory.I.GetPort(portName, portSettings, 10 * 1000);

                // send command to request firmware name
                byte[] firmwareNameRequestCommand = new byte[] { 0x1b, (byte)'#', (byte)'*', 0x0a, 0x00 };
                uint totalSizeCommunicated = WritePortHelper(port, firmwareNameRequestCommand);
                if (totalSizeCommunicated != firmwareNameRequestCommand.Length)
                {
                    Console.WriteLine("Error - could not write out all data.");
                    return;
                }
                
                char[] firmwareNameBuffer = new char[100]; // will never be this long
                int firmwareNameBufferIdx = 0;

                int wrongCount = 0;
                const int readState_init = 0, readState_inFirmwareName_1 = 1, readState_inFirmwareName_2 = 2, readState_inFirmwareName_3 = 3, readState_inFirmwareName_4 = 4, readState_inFirmwareName_5 = 5, readState_done = 6;
                int readState = readState_init;

                while ((readState != readState_done) && (wrongCount < 50))
                {
                    byte[] inputBuffer = new byte[1024];
                    uint readLength = port.ReadPort(ref inputBuffer, 0, (uint)inputBuffer.Length);
                    if (readLength == 0)
                    {
                        break;
                    }

                    for (int i = 0; i < readLength; i++)
                    {
                        if ((readState == readState_init) && (inputBuffer[i] == 0x1b))
                        {
                            readState = readState_inFirmwareName_1;
                        }
                        else if ((readState == readState_inFirmwareName_1) && (inputBuffer[i] == (byte)'#'))
                        {
                            readState = readState_inFirmwareName_2;
                        }
                        else if ((readState == readState_inFirmwareName_2) && (inputBuffer[i] == (byte)'*'))
                        {
                            readState = readState_inFirmwareName_3;
                        }
                        else if ((readState == readState_inFirmwareName_3) && (inputBuffer[i] == (byte)','))
                        {
                            readState = readState_inFirmwareName_4;
                        }
                        else if ((readState == readState_inFirmwareName_4) && (inputBuffer[i] == 0x0a))
                        {
                            readState = readState_inFirmwareName_5;
                        }
                        else if ((readState == readState_inFirmwareName_5) && (inputBuffer[i] == 0x00))
                        {
                            readState = readState_done;
                            break;
                        }
                        else if (readState == readState_inFirmwareName_4) // must be last before the catch-all
                        {
                            firmwareNameBuffer[firmwareNameBufferIdx++] = (char)inputBuffer[i];
                        }
                        else
                        {
                            readState = readState_init;
                            firmwareNameBufferIdx = 0;
                            wrongCount++;
                        }
                    }
                }

                if (readState == readState_done)
                {
                    Console.WriteLine("Success - firmware name = " + new string(firmwareNameBuffer));
                }
                else
                {
                    Console.WriteLine("Failure - could not read in the firmware name.");
                }
            }
            catch (PortException e)
            {
                Console.WriteLine("Failure - see stack trace:\n" + e.StackTrace);
            }
            finally
            {
                if (port != null)
                {
                    StarMicronics.StarIO.Factory.I.ReleasePort(port);
                }
            }
        }

        private static uint WritePortHelper(IPort port, byte[] writeBuffer)
        {
            uint zeroProgressOccurances = 0;
            uint totalSizeCommunicated = 0;
            while ((totalSizeCommunicated < writeBuffer.Length) && (zeroProgressOccurances < 2)) // adjust zeroProgressOccurances as needed
            {
                uint sizeCommunicated = port.WritePort(writeBuffer, totalSizeCommunicated, (uint)writeBuffer.Length - totalSizeCommunicated);
                if (sizeCommunicated == 0)
                {
                    zeroProgressOccurances++;
                }
                else
                {
                    totalSizeCommunicated += sizeCommunicated;
                    zeroProgressOccurances = 0;
                }
            }
            
            return totalSizeCommunicated;
        }
    }
}
