///////////////////////////////////////////////////////////////////////////
// Use these platform specific commands to compile and run this sample.
//
// Windows x86 Compile command: "javac -classpath bin_Win32/stario.jar -d bin_Win32 StarIO_Sample.java"
//             Execution command: "java -classpath bin_Win32;bin_Win32/stario.jar StarIO_Sample"
//
// Windows x64 Compile command: "javac -classpath bin_x64/stario.jar -d bin_x64 StarIO_Sample.java"
//             Execution command: "java -classpath bin_x64;bin_x64/stario.jar StarIO_Sample"
//
// Linux & Mac Compile command: "javac -classpath stario.jar StarIO_Sample.java"
//             Execution command: "java -classpath stario.jar:. StarIO_Sample"
//////////////////////////////////////////////////////////////////////////////

import com.starmicronics.stario.*;
import java.util.Scanner;

class StarIO_Sample
{
    static final int TIMEOUT = 10000;

    public static void main(String[] args)
    {
        Scanner in = new Scanner(System.in);
        
        System.out.print("Enter the port name: ");
        String portName = in.nextLine();

        System.out.print("Enter the port settings: ");
        String portSettings = in.nextLine();

        while(true)
        {
            System.out.println("[1] Write Port");
            System.out.println("[2] Retrieve Status");
            System.out.println("[3] Reset Device");
            System.out.println("[4] Checked Block");
            System.out.println("[5] Read Port");
            System.out.println("[x] Quit");
            System.out.print("Select one of the above options: ");
            String answerString = in.nextLine();

            if(answerString.compareTo("x") == 0)
            {
                return;
            }

            try
            {
                int answer = Integer.parseInt(answerString);
                
                switch(answer)
                {
                case 1:
                    WritePortUsage(portName, portSettings);
                    break;
                case 2:
                    RetrieveStatusUsage(portName, portSettings);
                    break;
                case 3:
                    ResetDeviceUsage(portName, portSettings);
                    break;
                case 4:
                    CheckedBlockUsage(portName, portSettings);
                    break;
                case 5:
                    ReadPortUsage(portName, portSettings);
                    break;
                default:
                    System.out.println("Invalid Option");
                    break;
                }
            }
            catch (NumberFormatException e)
            {
                System.out.println("Invalid Option");
            }
            
            System.out.println();
        }
    }
    
    public static void WritePortUsage(String portName, String portSettings)
    {
        StarIOPort port = null;
        try
        {
            port = StarIOPort.getPort(portName, portSettings, TIMEOUT);
            
            byte[] data = "Hello World!\n".getBytes();

            int totalSizeCommunicated = WritePortHelper(port, data); // see below
            if (totalSizeCommunicated == data.length)
            {
                System.out.println("Success");
            }
            else
            {
                System.out.println("Error - Could not write all data to the printer.");
            }
        }
        catch (StarIOPortException e)
        {
            System.out.println("Failure - see stack trace:");
            e.printStackTrace();
        }
        finally
        {
            if (port != null)
            {
                StarIOPort.releasePort(port);
            }
        }
    }
    
    public static void RetrieveStatusUsage(String portName, String portSettings)
    {
        StarIOPort port = null;
        try
        {
            port = StarIOPort.getPort(portName, portSettings, TIMEOUT);
        
            StarPrinterStatus starPrinterStatus = port.retreiveStatus();
            if (starPrinterStatus.offline == true)
            {
                if (starPrinterStatus.coverOpen == true)
                {
                    System.out.println("The printer is offline - cover open.");
                }
                else  if (starPrinterStatus.receiptPaperEmpty == true)
                {
                    System.out.println("The printer is offline - paper empty.");
                }
                else
                {
                    System.out.println("The printer is offline.");
                }
            }
            else
            {
                System.out.println("The printer is online.");
            }
            
            System.out.println("Success");
        }
        catch (StarIOPortException e)
        {
            System.out.println("Failure - see stack trace:");
            e.printStackTrace();
        }
        finally
        {
            if (port != null)
            {
                StarIOPort.releasePort(port);
            }
        }
    }
    
    public static void ResetDeviceUsage(String portName, String portSettings)
    {
        StarIOPort port = null;
        try
        {
            port = StarIOPort.getPort(portName, portSettings, TIMEOUT);
        
            port.resetDevice();
        }
        catch (StarIOPortException e)
        {
            System.out.println("Failure - see stack trace:");
            e.printStackTrace();
        }
        finally
        {
            if (port != null)
            {
                StarIOPort.releasePort(port);
            }
        }
    }
    
    public static void CheckedBlockUsage(String portName, String portSettings)
    {
        StarIOPort port = null;
        try
        {
            port = StarIOPort.getPort(portName, portSettings, TIMEOUT);
            
            StarPrinterStatus starPrinterStatus;
            
            starPrinterStatus = port.beginCheckedBlock();
            if(starPrinterStatus.offline == true)
            {
                System.out.println("The printer is offline, so printing will not proceed.");
                return;
            }

            byte[] data = "Hello World!\n".getBytes();
            for(int count = 0; count < 50; count++)
            {
                int totalSizeCommunicated = WritePortHelper(port, data); // see below
                if (totalSizeCommunicated != data.length)
                {
                    System.out.println("Error - Could not write all data to the printer.");
                    return;
                }
            }
			
            starPrinterStatus = port.endCheckedBlock();
            if (starPrinterStatus.offline == true)
            {
                System.out.println("The printer is offline, meaning that printing failed.");
            }
            
            System.out.println("Success.");
        }
        catch (StarIOPortException e)
        {
            System.out.println("Failure - see stack trace:");
            e.printStackTrace();
        }
        finally
        {
            if (port != null)
            {
                StarIOPort.releasePort(port);
            }
        }
    }
    
    public static void ReadPortUsage(String portName, String portSettings)
    {
        StarIOPort port = null;
        try
        {
            port = StarIOPort.getPort(portName, portSettings, TIMEOUT);
            
            byte[] firmwareNameRequestCommand = new byte[]{ 0x1b, (byte)'#', (byte)'*', 0x0a, 0x00 };
            int totalSizeCommunicated = WritePortHelper(port, firmwareNameRequestCommand); // see below
            if (totalSizeCommunicated != firmwareNameRequestCommand.length)
            {
                System.out.println("Error - Could not write all data to the printer.");
                return;
            }
            
            char[] firmwareNameBuffer = new char[100];
            int firmwareNameBufferIdx = 0;
            int tryCount = 0;
            
            int wrongCount = 0;
            final int readState_init = 0, readState_inFirmwareName_1 = 1, readState_inFirmwareName_2 = 2, readState_inFirmwareName_3 = 3, readState_inFirmwareName_4 = 4, readState_inFirmwareName_5 = 5, readState_done = 6;
            int readState = readState_init;
            
            while ((readState != readState_done) && (wrongCount < 50))
            {
                byte[] inputBuffer = new byte[1024];
                int readLength = port.readPort(inputBuffer, 0, inputBuffer.length);
                if (readLength == 0)
                {
                    tryCount++;
                    if (tryCount > 2)
                    {
                        break;
                    }
                }
                
                for (int i = 0; i < readLength; i++)
                {
                    if ((readState == readState_init) && (inputBuffer[i] == 0x1b))
                    {
                        readState = readState_inFirmwareName_1;
                    }
                    else if ((readState == readState_inFirmwareName_1) && (inputBuffer[i] == (byte)'#'))
                    {
                        readState = readState_inFirmwareName_2;
                    }
                    else if ((readState == readState_inFirmwareName_2) && (inputBuffer[i] == (byte)'*'))
                    {
                        readState = readState_inFirmwareName_3;
                    }
                    else if ((readState == readState_inFirmwareName_3) && (inputBuffer[i] == (byte)','))
                    {
                        readState = readState_inFirmwareName_4;
                    }
                    else if ((readState == readState_inFirmwareName_4) && (inputBuffer[i] == 0x0a))
                    {
                        readState = readState_inFirmwareName_5;
                    }
                    else if ((readState == readState_inFirmwareName_5) && (inputBuffer[i] == 0x00))
                    {
                        readState = readState_done;
                        break;
                    }
                    else if (readState == readState_inFirmwareName_4) // must be last before the catch-all
                    {
                        firmwareNameBuffer[firmwareNameBufferIdx++] = (char)inputBuffer[i];
                    }
                    else
                    {
                        readState = readState_init;
                        firmwareNameBufferIdx = 0;
                        wrongCount++;
                    }
                }
            }
            
            if (readState == readState_done)
            {
                System.out.println("Success - firmware name = " + new String(firmwareNameBuffer));
            }
            else
            {
                System.out.println("Failure - could not read in the firmware name.");
            }
        }
        catch (StarIOPortException e)
        {
            System.out.println("Failure - see stack trace:");
            e.printStackTrace();
        }
        finally
        {
            if (port != null)
            {
                StarIOPort.releasePort(port);
            }
        }
    }
    
    private static int WritePortHelper(StarIOPort port, byte[] writeBuffer) throws StarIOPortException
    {
        int zeroProgressOccurances = 0;
        int totalSizeCommunicated = 0;
        while ((totalSizeCommunicated < writeBuffer.length) && (zeroProgressOccurances < 2)) // adjust zeroProgressOccurances as needed
        {
            int sizeCommunicated = port.writePort(writeBuffer, totalSizeCommunicated, writeBuffer.length - totalSizeCommunicated);
            if (sizeCommunicated == 0)
            {
                zeroProgressOccurances++;
            }
            else
            {
                totalSizeCommunicated += sizeCommunicated;
                zeroProgressOccurances = 0;
            }
        }
        
        return totalSizeCommunicated;
    }
}
